@@ -1,6 +1,6 @@
 #!/usr/bin/env python3.7
 
-#  What does this script do?
+# What does this script do?
 # This script takes a pair of read (R1 and R2) in the format of dataframe row
 # and call mutaitons based on the reads
 
@@ -11,21 +11,30 @@ from Bio.Seq import Seq
 
 from TileSeqMut import posterior
 pd.options.mode.chained_assignment = None  # default='warn'
-# import posterior
 
 class MutParser(object):
 
     def __init__(self, row, full_seq, cds_seq, seq_lookup, tile_s, tile_e, post_prob_cutoff, logging, mut_rate, base,
-                 posteriorQC, adjusted_er):
+                 posteriorQC, error_override, adjusted_er):
         """
-        row: input row includes both reads from sam file
-        full_seq: full sequence (including cds and padding sequence)
-        cds_seq: coding sequence (includes stop codon)
-        seq: read from json file - seq, cds_start, cds_end
+        @param row: input row includes both reads from sam file
+        @param full_seq: full sequence (including cds and padding sequence)
+        @param cds_seq: coding sequence (includes stop codon)
+        @param seq_lookup: df contains coding sequences and protein sequence
+        @param tile_s: tile start
+        @param tile_e: tile end
+        @param post_prob_cutoff:
+        @param logging:
+        @param mut_rate: mut rate defined by user
+        @param base:
+        @param posterorQC: boolean, if posterior QC required
+        @param error_override: boolean, if error probabilities should be obtained from observed error
+        @param adjusted_er:
         """
+
         self._seq = Seq(full_seq["seq"].values.item())
         self._cds = Seq(cds_seq)
-        self._start_pos = full_seq.cds_start.values.item() # 1 posisiton
+        self._start_pos = full_seq.cds_start.values.item()  # 1 posisiton
         self._end_pos = full_seq.cds_end.values.item()
         self._tile_begins = tile_s
         self._tile_ends = tile_e
@@ -44,7 +53,8 @@ class MutParser(object):
         self._base = base
         self._posteriorQC = posteriorQC
         
-        # if we are adjusting phred scores 
+        # if we are adjusting phred scores
+        self._error_override = error_override 
         self._adjusted_er = adjusted_er
 
     def _get_seq(self):
@@ -55,10 +65,9 @@ class MutParser(object):
         # get information for R1
         # where the mapping starts
         self._r1_pos = int(self._reads["pos_start_r1"])
-        # self._r1_cigar = list(cigar.Cigar(self._reads["cigar_r1"]).items())
         r1_cigar = [i for i in re.split("([\d]+\S)", self._reads["cigar_r1"]) if i != ""]
         self._r1_cigar = [(re.split(r"([\d]+)", i)[1], re.split(r"([\d]+)", i)[2]) for i in r1_cigar]
-        self._r1_readlen = sum([int(i[0]) for i in self._r1_cigar])
+        # self._r1_readlen = sum([int(i[0]) for i in self._r1_cigar])
         self._r1_ref = self._seq[int(self._r1_pos)-1:int(self._r1_pos)+len(self._reads["seq_r1"])]
         self._r1_qual = self._reads["qual_r1"]
         self._r1_read = self._reads["seq_r1"]
@@ -66,24 +75,25 @@ class MutParser(object):
 
         # get the ref sequence for R2
         self._r2_pos = int(self._reads["pos_start_r2"])
-        # self._r2_cigar = list(cigar.Cigar(self._reads["cigar_r2"]).items())
         r2_cigar = [i for i in re.split("([\d]+\S)", self._reads["cigar_r2"]) if i != ""]
         self._r2_cigar = [(re.split(r"([\d]+)", i)[1], re.split(r"([\d]+)", i)[2]) for i in r2_cigar]
-        self._r2_readlen = sum([int(i[0]) for i in self._r2_cigar])
+        # self._r2_readlen = sum([int(i[0]) for i in self._r2_cigar])
         self._r2_ref = self._seq[int(self._r2_pos)-1:int(self._r2_pos)+len(self._reads["seq_r2"])]
         self._r2_qual = self._reads["qual_r2"]
         self._r2_read = self._reads["seq_r2"]
         self._r2_mdz = self._reads["mdz_r2"]
 
-        return self
-
     def _convert_mut(self, mut_list, read):
         """
         convert a list of mutations into the input format for posterior function
+        @param mut_list: list of mutations
+        @param read: read name (read 1 or read 2)
         return a df with mutations
         """
         if mut_list == []:
             return pd.DataFrame({}, columns=["m", "read", "pos", "ref", "alt", "qual"])
+
+        # flatten deletions
         final_mut = []
         for i in mut_list:
             mut = i.split("|")
@@ -95,6 +105,7 @@ class MutParser(object):
                     final_mut.append(m)
             else:
                 final_mut.append(i)
+
         final_df = pd.DataFrame({"m":final_mut})
         final_df["read"] = read
         final_df[["pos", "ref", "alt", "qual"]] = final_df["m"].str.split("|", expand=True)
@@ -122,17 +133,19 @@ class MutParser(object):
 
         # parse mutations in R1
         r1_snp, r1_delins, map_pos_r1 = self._parse_cigar_mdz(self._r1_cigar, self._r1_mdz, self._r1_ref, self._r1_read,
-                                                 self._r1_pos,self._r1_qual)
+                                                 self._r1_pos, self._r1_qual)
         # parse mutations in R2
         r2_snp, r2_delins, map_pos_r2 = self._parse_cigar_mdz(self._r2_cigar, self._r2_mdz, self._r2_ref, self._r2_read,
-                                                  self._r2_pos,self._r2_qual)
-
+                                                  self._r2_pos, self._r2_qual)
+        # convert snps to df
         final_df_r1 = self._convert_mut(r1_snp, "r1")
         final_df_r2 = self._convert_mut(r2_snp, "r2")
 
         # merge two df
         snp_df = pd.merge(final_df_r1, final_df_r2, on=["pos"], how="outer", suffixes=('_r1', '_r2'))
         snp_df["pos"] = pd.to_numeric(snp_df["pos"])
+
+        # convert delins to df
         final_df_r1 = self._convert_mut(r1_delins, "r1")
         final_df_r2 = self._convert_mut(r2_delins, "r2")
 
@@ -140,9 +153,10 @@ class MutParser(object):
         delins_df = pd.merge(final_df_r1, final_df_r2, on=["pos"], how="outer", suffixes=('_r1', '_r2'))
         delins_df["pos"] = pd.to_numeric(delins_df["pos"])
 
-        merged_df = [snp_df, delins_df]
-        merged_df = pd.concat(merged_df)
+        merged_df = pd.concat([snp_df, delins_df])
+        # sort value by position
         merged_df = merged_df.sort_values(by="pos")
+        # group by position
         merged_df = merged_df.groupby("pos").first().reset_index()
         # build df to track how many mutations were rejected 
         track_df = merged_df[["pos", "m_r1", "m_r2"]]
@@ -157,12 +171,24 @@ class MutParser(object):
         d = dict(tuple(merged_df.groupby(merged_df['pos'].diff().gt(n).cumsum())))
         # analyze the dictionary of clusters
         # and get posterior
-        # print(d, self._mutrate, self._cutoff)
-        pos_df, all_df, clustered_r1, clustered_r2 = posterior.cluster(d, self._r1_qual,self._r2_qual, map_pos_r1,
+        pos_df, wt_df, all_df, clustered_r1, clustered_r2 = posterior.cluster(d, self._r1_qual, self._r2_qual, map_pos_r1,
                                                                  map_pos_r2, self._mutrate, self._cutoff, self._base,
-                                                                       self._posteriorQC, adjustthred=self._adjusted_er)
+                                                                       self._posteriorQC, self._error_override, self._adjusted_er)
         final_mut = list(set(pos_df.m.tolist()))
         final_mut.sort()
+        wt_calls = list(set(wt_df.m.tolist()))
+        wt_calls.sort()
+
+    # create wt_df to store positions where WT call passed filter cut-off
+        if wt_calls != []:
+            wt_df = pd.DataFrame([sub.split("|") for sub in wt_calls])
+            wt_df = wt_df.rename(columns=str).rename(columns={"0": "pos", "1": "wt_passed"})
+            wt_df = wt_df[["pos", "wt_passed"]]
+            wt_df["pos"] = wt_df["pos"].astype(int)
+            wt_df["pos"] = wt_df["pos"] - self._start_pos + 1
+        else:
+            wt_df = pd.DataFrame(columns=['pos', 'wt_passed'])
+
         if final_mut != []:
             final_df = pd.DataFrame([sub.split("|") for sub in final_mut])
             # give the position col a name
@@ -170,13 +196,21 @@ class MutParser(object):
             final_df = final_df[["pos", "passed"]]
             final_df["pos"] = final_df["pos"].astype(int)
             final_df["pos"] = final_df["pos"] - self._start_pos + 1
+
             # merge final df with track df to see how many mutations passed filter on each/both reads
             merged_track_df = pd.merge(track_df, final_df, how="left", on="pos")
+            # merge final df to see how many mutation could confidently be called WT
+            merged_track_df = pd.merge(merged_track_df, wt_df, how="left", on="pos")
+
             merged_track_df["passed"] = merged_track_df["passed"].where(~merged_track_df["passed"].notna(), 1)
             merged_track_df["passed"] = merged_track_df["passed"].fillna(0)
+            merged_track_df["wt_passed"] = merged_track_df["wt_passed"].where(~merged_track_df["wt_passed"].notna(), 1)
+            merged_track_df["wt_passed"] = merged_track_df["wt_passed"].fillna(0)
         else:
             merged_track_df = track_df
             merged_track_df["passed"] = 0
+            merged_track_df["wt_passed"] = 0
+
         hgvs_r1_clusters, outside_mut_r1 = [], []
         hgvs_r2_clusters, outside_mut_r2 = [], []
         if not clustered_r1.empty:
@@ -200,68 +234,73 @@ class MutParser(object):
     def _parse_cigar_mdz(self, cigar, mdz_raw, ref, read, pos, qual):
         """
         use CIGAR string and mdz tag to call mutations from a read
+        @param: cigar: CIGAR string in a list (i.e [("149", "M"), ("1", "S")]
+        @param: mdz_raw: MDZ raw str, (i.e 39G2A14A21A14C4T9A1T7T11C4C13)
+        @param: read: input read sequence
+        @param: pos: pos of read mapped to 
+        @param: qual: quality str of the read
+        return:  snp_list, delins_list, pos_map
+
         """
-        snp_list = [] # output list with all the mutations
+        # output list with all the mutations
+        # track snp and del/ins separately
+        snp_list = []
         delins_list = []
-        # soft clip
+        # adjust for soft clip
         clip = 0
         # check starting point of the sequence
         # remove soft clipped bases and adjust for position
-        if cigar[0][1] == "S": # soft clip occurs in the beginning of a read
+        # soft clip occurs in the beginning of a read
+        if cigar[0][1] == "S":
             clip += int(cigar[0][0])
-
         # convert mdz string into a list
+        # i.e ['5G', '41C', '4G', '11A', '7G', '5G', '10C', '30G']
         mdz = re.findall('.*?[.ATCG]+', mdz_raw)
-        # convert cigar list to a string
-        cigar_joined = [str(i[0])+str(i[1]) for i in cigar]
-
         ins_pos_ref = 0 # on ref sequence where was the insertion
-        total_len = 0 # total lenth indicates in CIGAR string
         ins_pos = [] # store [ins_pos, ins_len]
-
         mapped = 0
         deleted = 0
-
-        # create a dictionary to store the mapped positions (ref_pos:read_pos)
-        # this is later used to get the quality score for a base
-        map_pos = {}
+        # make two list: positions on ref sequence and pos on read
+        # merge this two lists into a dictionary later
         ref_positions = []
         read_positions = []
+        # read starts after soft clip
         read_start = 0+clip
+        # ref starts at pos 
         ref_start = pos
-        #if "I" in "".join(cigar_joined): # if insertion is in the read
-            # get insertion position from cigar string
+        # get insertion position from cigar string
         for i in cigar: # go through each cigar string
             if i[1] != "I": # not insertion
                 ins_pos_ref += int(i[0])
                 if i[1] == "M": # track number of bases mapped
                     ref_positions += list(range(ref_start, ref_start+int(i[0])))
                     read_positions += list(range(read_start, read_start+int(i[0])))
+                    # shift ref to the right
                     ref_start += int(i[0])
+                    # shift read to the right
                     read_start += int(i[0])
+                    # track number of bases mapped
                     mapped += int(i[0])
-
                 if i[1] == "D": # track number of bases deleted
                     deleted += int(i[0])
-                    #read_start += int(i[0])
                     # skip deleted bases on ref read
+                    # no need to skip on read bacause it's deleted
                     ref_start += int(i[0])
-
             else:
                 # based on number of bases mapped, get the inserted base from read
                 ins_base = read[read_start:read_start+int(i[0])]
-                qual_base = qual[read_start:read_start + int(i[0])]
+                qual_base = qual[read_start:read_start+int(i[0])]
                 qual_base = ",".join(list(qual_base))
-                # calculate inserted base posterior by using the quality string
-                # qual_ins = qual[mapped:mapped+int(i[0])]
                 # keep the insertion position and inserted lenth in a list
                 ins_pos.append([ins_pos_ref, int(i[0])])
                 # add the insertion to mut_list
-                #mut_list.append([str(pos+mapped+deleted),ins_base,"ins"])
+                # format: inserted position|base(s) inserted|ins|quality of the inserted base(s)
                 delins_list.append(f"{pos + mapped + deleted}|{ins_base}|ins|{qual_base}")
                 # skip inserted bases on read
                 read_start += int(i[0])
+
         # zip two lists into a dictionary
+        # this mapped reference pos to read pos
         pos_map = dict(zip(ref_positions, read_positions))
         # parse snp and deletion from mdz string
         # given the inserted position on reference sequence
@@ -274,7 +313,6 @@ class MutParser(object):
         inserted_pos = 0
         deleted_len = 0
         map_pos = 0 + clip
-
         for i in mdz:
             # for each item in the MD:Z string
             # split the item into number and letter
@@ -282,7 +320,7 @@ class MutParser(object):
             match_len = int(m.group(1))
             base = m.group(2)
 
-            map_pos += match_len# update how many bp are mapped
+            map_pos += match_len  # update how many bp are mapped
 
             while ins and map_pos >= ins[0]:
                 inserted_pos += ins[1]
@@ -291,45 +329,26 @@ class MutParser(object):
             if "^" not in base:
                 read_pos += match_len
                 # this means a single nt change
-                # mut_list.append([str(pos+read_pos-clip),base,read[read_pos+inserted_pos-deleted_len],
-                # qual[read_pos+inserted_pos-deleted_len]])
                 pos_on_read = read_pos+inserted_pos-deleted_len+clip
+                # format: position of snp|reference|alt|quality
                 snp_list.append(f"{pos+read_pos}|{base}|{read[pos_on_read]}|{qual[pos_on_read]}")
-
                 # adjust read pos with 1bp change (move to the right for 1 pos)
                 map_pos += len(base)
-                read_pos += 1
+                read_pos += len(base)
+
             else: # deletion
                 read_pos += match_len
-                #mut_list.append([str(pos+read_pos-clip),base[1:],"del"])
                 # quality of the base before deleted base
                 qual_pre = qual[read_pos+inserted_pos-deleted_len+clip-1]
                 # quality of the base after deleted base
                 qual_post = qual[read_pos+inserted_pos-deleted_len+clip]
-                delins_list.append(f"{pos + read_pos}|{base[1:]}|del|{qual_pre},{qual_post}")
-
+                # format: position|deleted base(s)|del|qual score pre to this pos, qual score post to this pos
+                delins_list.append(f"{pos+read_pos}|{base[1:]}|del|{qual_pre},{qual_post}")
+                # adjust bases 
                 deleted_len += len(base[1:])
                 read_pos += len(base[1:])
                 map_pos += len(base[1:])
 
-        # if "287|T|C|E" in snp_list:
-        #     print(snp_list, delins_list)
-        #     print(self._r1_pos,
-        #     self._r1_cigar,
-        #     self._r1_readlen,
-        #     self._r1_ref,
-        #     self._r1_qual,
-        #     self._r1_read ,
-        #     self._r1_mdz)
-        #
-        #     print(self._r2_pos,
-        #           self._r2_cigar,
-        #           self._r2_readlen,
-        #           self._r2_ref,
-        #           self._r2_qual,
-        #           self._r2_read,
-        #           self._r2_mdz)
-
         return snp_list, delins_list, pos_map
 
     def _get_hgvs(self, mut_list):
@@ -350,7 +369,7 @@ class MutParser(object):
         # also record any snp that happened in the range
         delins = []
         mutations = []
-
+        
         for mut in mut_list:
             mut_change = mut.split("|")
             tmp_pos = int(mut_change[0]) # template position
@@ -400,10 +419,7 @@ class MutParser(object):
                         mutations.append(hgvs)
 
             else: # snp
-                #mut_change = mut.split("|")
-                #tmp_pos = int(mut_change[0])
                 # get cds position from look up table
-                #cds_pos = self._seq_lookup[self._seq_lookup.temp_pos == tmp_pos].cds_pos.item()
                 cds_ref = self._cds[cds_pos-1]
 
                 ## validate: if the reference base matches the ref in seq_lookup
@@ -418,7 +434,7 @@ class MutParser(object):
                     if cds_pos == concecutive_snp[-1] +1:
                         # update position in concec_pos
                         combined_snp+=mut_change[2]
-                        # update basesin combined_bases
+                        # update bases in combined_bases
                         concecutive_snp.append(cds_pos)
                     elif cds_pos == concecutive_snp[-1] +2:
                         # get middle ref
@@ -447,42 +463,6 @@ class MutParser(object):
             joined = ";".join(mutations)
             mutations = f"c.[{joined}]"
 
-        # output_df = pd.DataFrame(output_df, columns=["mut", "hgvs"])
-        # print(output_df)
-
-        #if "c.542del" in mutations:
-         #   print(self._r1_pos,
-         #   self._r1_cigar,
-         #   self._r1_readlen,
-         #   self._r1_ref,
-         #   self._r1_qual,
-         #   self._r1_read ,
-         #   self._r1_mdz)
-         #   print(self._r2_pos,
-         #   self._r2_cigar,
-         #   self._r2_readlen,
-         #   self._r2_ref,
-         #   self._r2_qual,
-         #   self._r2_read ,
-         #   self._r2_mdz)
-         #   print("------")
-        #if "c.596del" in mutations:
-        #    print(self._r1_pos,
-        #    self._r1_cigar,
-        #    self._r1_readlen,
-        #    self._r1_ref,
-        #    self._r1_qual,
-        #    self._r1_read ,
-        #    self._r1_mdz)
-        #    print(self._r2_pos,
-        #    self._r2_cigar,
-        #    self._r2_readlen,
-        #    self._r2_ref,
-        #    self._r2_qual,
-        #    self._r2_read ,
-        #    self._r2_mdz)
-        #    print("------")
-
         return mutations, outside_mut
 
 
@@ -556,7 +536,6 @@ def delins_to_hgvs(cds_seq, delins):
                     prev_pos += len(mid)
 
         hgvs = f"{start_pos}_{end_pos}delins{modified}"
-        #print("del+ins, ",hgvs)
     return hgvs
 
 
@@ -566,7 +545,6 @@ class DuplicatedMutations(Exception):
         Attributes:
             message -- explanation of the error
         """
-
     def __init__(self, message="Same position mutated twice in one read!"):
         self.message = message
         super().__init__(self.message)
@@ -578,7 +556,6 @@ class SameRefandAlt(Exception):
         Attributes:
             message -- explanation of the error
         """
-
     def __init__(self, mut_df, message="Same reference base and alt base!"):
         self.mut = mut_df
         self.message = message
@@ -587,6 +564,7 @@ class SameRefandAlt(Exception):
     def __str__(self):
         return f'{self.mut} -> {self.message}'
 
+
 if __name__ == "__main__":
 
     # test delins_to_hgvs
